
/*
 * $Id: ILog.cc,v 1.5.6.1 1999/12/11 23:40:56 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ILog.h"
#include "gadgets.h"
#include "polyLogTags.h"

ILog::ILog(): theStream(0), theEntryEnd(-1) {
}

ILog::~ILog() {
}

void ILog::stream(const String &aFileName, istream *aStream) {
	theFileName = aFileName;
	theStream = aStream;
	getHeader();
}

// XXX: make this a method of LogEntryPx
// XXX: we should log lgEnd instead of using current constant!
inline
bool goodPx(const LogEntryPx &px) {
	return px.theTag >= 0 && px.theTag < lgEnd &&
		px.theSize <= 10*1024*1024;
}

// loads prefix of the current entry
// transparently loads progress entries
LogEntryPx ILog::begEntry() {
	theEntryEnd = theStream->tellg();
	while ((*this) >> theCurPx.theSize >> theCurPx.theTag) {

		if (theCurPx && !goodPx(theCurPx)) {
			const streampos startOff = theEntryEnd;
			clog << fileName() << ':' << theEntryEnd 
				<< ": warning: corrupted log entry: tag: " << theCurPx.theTag << " size: " << theCurPx.theSize << endl;

			do {
				theEntryEnd++;
				endEntry();
				(*this) >> theCurPx.theSize >> theCurPx.theTag;
			} while (*theStream && !(theCurPx.theTag == lgProgress && theCurPx.theSize > 0 && theCurPx.theSize < 1024));
			if (!*theStream)
				break;

			clog << fileName() << ':' << theEntryEnd
				<< ": maybe recovered after skipping " << (theEntryEnd-startOff)
				<< " bytes; tag: " << theCurPx.theTag << " size: " << theCurPx.theSize << endl;
		}

		theEntryEnd += theCurPx.theSize;
		if (theCurPx.theTag != lgProgress)
			return theCurPx;
		theProgress.load(*this);
		endEntry();
	}
	theCurPx.theSize = 0;
	theCurPx.theTag = 0;
	return theCurPx;
}

void ILog::endEntry() {
	theStream->seekg(theEntryEnd);
	theCurPx.theSize = 0;
	theCurPx.theTag = 0;
}

void ILog::getHeader() {
	Assert(theStream);

	// check magic
	if (geti() != lgMagic1 || geti() != lgMagic2 || geti() != 0) {
		if (theStream->bad())
			cerr << theFileName << ": read error; " << error(errno) << endl;
		else
			cerr << theFileName << ": unknown log file format" << endl;
		exit(-2);
	}

	const int sver = 2;      // supported version
	const int cver = geti(); // current log version
	const int rver = geti(); // required min version to support
	const int skip = geti(); // size of extra headers

	if (rver > sver) {
		cerr << theFileName << ": log version: " << cver << endl;
		cerr << theFileName << ": requires support for log version " << rver << " or higher; this program supports version " << sver << endl;
		exit(-2);
	}

	theStream->ignore(skip);
}

int ILog::geti(int *&xs, int &count) {
	Must(geti(count) >= 0);
	xs = new int[count];
	for (int i = 0; i < count; ++i)
		geti(xs[i]);
	return count;
}

String &ILog::gets(String &s) {
	const int sz = geti();
	Must(sz >= 0);
	s.alloc(sz);
	if (sz)
		get(s.buf(), sz);
	return s;
}
