
/*
 * $Id: BcastRcver.h,v 1.5.4.1 1999/10/22 21:41:51 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_BROADCAST_RECEIVER_H
#define PG_BROADCAST_RECEIVER_H

#include "Array.h"

class BcastChannel;
class Connection;
class Xaction;
class OLog;

// broadcast channel(s) subscriber

class BcastRcver {
	public:
		typedef enum { ieNone, ieWorkSetFreeze } InfoEvent;
	public:
		virtual ~BcastRcver();

		inline void noteEvent(BcastChannel *ch, const Connection *c);
		inline void noteEvent(BcastChannel *ch, const Xaction *x);
		inline void noteEvent(BcastChannel *ch, const char *msg);
		inline void noteEvent(BcastChannel *ch, int errNo);
		inline void noteEvent(BcastChannel *ch, InfoEvent ev);
		inline void noteEvent(BcastChannel *ch, OLog *log);

		void startListen();
		void stopListen();

	protected:
		virtual void noteConnEvent(BcastChannel *ch, const Connection *c);
		virtual void noteXactEvent(BcastChannel *ch, const Xaction *x);
		virtual void noteMsgStrEvent(BcastChannel *ch, const char *msg);
		virtual void noteErrNoEvent(BcastChannel *ch, int errNo);
		virtual void noteInfoEvent(BcastChannel *ch, InfoEvent ev);
		virtual void noteLogEvent(BcastChannel *ch, OLog &log);

	protected:
		Array<BcastChannel*> theChannels; // channels we are subscribed to
};


/* inlined methods */

inline
void BcastRcver::noteEvent(BcastChannel *ch, const Connection *c) {
	noteConnEvent(ch, c);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, const Xaction *x) {
	noteXactEvent(ch, x);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, const char *msg) {
	noteMsgStrEvent(ch, msg);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, int errNo) {
	noteErrNoEvent(ch, errNo);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, InfoEvent ev) {
	noteInfoEvent(ch, ev);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, OLog *log) {
	noteLogEvent(ch, *log);
}

#endif
