
/*
 * $Id: DLog.h,v 1.8.6.1 1999/11/20 18:10:56 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_DIGEST_LOG_CLASS_H
#define PG_DIGEST_LOG_CLASS_H

#include "Array.h"
#include "PhaseLx.h"
#include "ErrorLx.h"
#include "Progress.h"
#include "IntvlCut.h"

class String;

// Digest Log is a collection of "interesting" info extracted
// from one phase of a binary log(s)
// XXX: rewrite the whole thing

class DLog {
	public:
		typedef void (*Filter)(const DLog &);

	public:
		DLog();
		~DLog();

		void filter(Filter aFilter);

		void noteRec(const StatPhaseRec &rec);
		void noteRec(const StatIntvlRec &rec);

		void noteErrorStrs(const ErrorStrTbl &est);
		void noteErrorStat(const ErrorStat &es);

	protected:
		void pushAccStats();

	public:
		String thePhaseName;

		Progress theProgress;     // progress since start of the current log
		IntervalCutter theCutter; // measures intervals to accumulate stats

		Array<String*> theSources; // how this log was built

	public:
		IntvlLxRec theCycleLxRec;
		PhaseLxRec thePhaseLxRec;
		ErrorLxRec theErrorLxRec;

	protected:
		Filter theFilter;      // gets called for every new 'cycle' record
};

#endif
