
/*
 * $Id: LogExtractor.h,v 1.3.6.1 1999/11/20 18:10:59 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_LOG_EXTRACTOR_H
#define PG_LOG_EXTRACTOR_H

#include "Array.h"
#include "String.h"

class ostream;
class StatIntvlRec;
class StatCycleRec;
class StatPhaseRec;
class ErrorStat;
class ErrorStrTbl;

// an amorphic thingy with some type checking
// helps build uniform-interface extractors independant of actual entry type
class LxRec {
	public:
		virtual ~LxRec();

		virtual const StatIntvlRec &intvl() const;
		virtual const StatCycleRec &cycle() const;
		virtual const StatPhaseRec &phase() const;
		virtual const ErrorStat &errorStat() const;

	protected:
		virtual const char *type() const = 0;
		void panic(const String &violator) const;
};

// parameters to configure extractors
class LxParams: protected Array<String*> {
	public:
		LxParams(const String &src);
		~LxParams();

		int count() const;
		const String &image() const;
		const String &operator [](int idx) const;

		void set(const String &src);

	protected:
		String theImage;
};

inline ostream &operator <<(ostream &os, const LxParams &p) { return os << p.image(); }

// extracts one object from an entry
class LogExtractor {
	public:
		LogExtractor();
		virtual ~LogExtractor();

		const String &name() const { return theName; }
		virtual void label(ostream &os);

		virtual bool configure(const String &aName, const LxParams &params);

		virtual void extract(const LxRec &r, ostream &os) = 0;

	protected:
		void checkParams(const LxParams &params, int count);
		void badParam(const LxParams &params, int idx, const String &exp) const;

	protected:
		String theName;
		String theLabel;
};

#endif
