
/*
 * $Id: Tuner.h,v 1.3 1999/09/07 17:47:24 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef TUNER_CLASS_H
#define TUNER_CLASS_H

// output_ratio = f(input_ratio)
// Function "f" is unknown or complex
// Output_ratio is measured, input_ratio is controlled,
// target_ratio is given.
// Tuner makes output_ratio close to target_ratio by adjusting 
// the input_ratio

class Tuner {
	public:
		Tuner();

		void sensitivity(double aSensy) { theSensy = aSensy; }
		void reactivity(double aReact) { theReact = aReact; }
		void target(double aTarget);
		void window(int aWin) { theWin = aWin; }

		double target() const { return theTarget; }

		void output(double or) { if (++theCnt >= theWin) syncInput(or); }
		void incOutput(int i) { theAcc += i; if (++theCnt >= theWin) syncInput(theAcc/(double)theCnt); }

		double input() const { return theInput; }  // read  this
		double output() const { return theOutput; }

	protected:
		void syncInput(double or);

	protected:
		double theSensy;  // sensistivity to new data [0,1]
		double theReact;  // how fast to change the input [0,1]
		double theTarget; // what output should be
		double theInput;  // our current best guess about the input
		double theOutput; // average output value

		int theWin;       // how often to sync the input in
		int theCnt;       // calls to output
		int theAcc;       // accumulates: output=acc/cnt
};

#endif
