
/*
 * $Id: HostsBasedIter.cc,v 1.1.2.2 1999/11/08 06:07:08 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"
#include "HostsBasedIter.h"

#include "AgentSym.h"
#include "PglNetAddrSym.h"


HostsBasedIter::HostsBasedIter(): theCfgIdx(0), theHostIdx(0) {
}

HostsBasedIter::HostsBasedIter(const Addrs *aLocalAddrs) {
	start(aLocalAddrs);
}

HostsBasedIter::~HostsBasedIter() {
}

HostsBasedIter &HostsBasedIter::start(const Addrs *aLocalAddrs) {
	theLocalAddrs = aLocalAddrs;

	theCfgIdx = theHostIdx = 0;
	sync();
	return *this;
}

bool HostsBasedIter::eof() const {
	return theCfgIdx >= cfgCount();
}

HostsBasedIter::operator void *() const {
	return !eof() ? (void*)(-1) : 0;
}

HostsBasedIter &HostsBasedIter::operator ++() {
	theHostIdx++;
	sync();
	return *this;
}

const NetAddr &HostsBasedIter::host() {
	Assert(!eof());
	return theHost;
}

NetAddr HostsBasedIter::host(int cidx, int hidx) const {
	return cfgSym(cidx)->host(hidx)->val();
}

// must be called after an index change
void HostsBasedIter::sync() {
	while (!eof()) {
		HostsBasedSym *cs = cfgSym(theCfgIdx);
		if (theHostIdx < cs->hostCount()) {
			if (belongs(cs, host(theCfgIdx, theHostIdx)))
				break;
			theHostIdx++;
		} else {
			theHostIdx = 0;
			theCfgIdx++;
		}
	}
		
	theHost = eof() ? NetAddr() : host(theCfgIdx, theHostIdx);
}

bool HostsBasedIter::belongs(const HostsBasedSym *, const NetAddr &host) const {
	if (!theLocalAddrs)
		return true; // no restrictions

	int i = theLocalAddrs->count();
	while (--i >= 0) {
		if (theLocalAddrs->item(i) == host.addrN())
			return true;
	}
	return false;
}
