
/*
 * $Id: PglRec.cc,v 1.5 1999/09/07 17:47:47 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "PglRec.h"
#include "PglRecSym.h"


PglRec::PglRec() {
}

PglRec::~PglRec() {
}

SynSymTblItem **PglRec::valp(const String &key) {
	// we are responsible for the first component of a dot-name only
	String ancor = key;
	String tail = 0;
	if (const char *p = key.chr('.')) {
		ancor = key(0, p-key);
		tail = p+1;
	}

	// lookup ancor
	if (SynSymTblItem **ip = SynSymTbl::valp(ancor)) {
		SynSymTblItem *i = *ip;
		Assert(i);

		// check if we need to go further
		if (tail) {
			if (i->sym() && i->sym()->isA("Rec")) {
				RecSym &s = (RecSym &) i->sym()->cast("Rec");
				return s.rec()->valp(tail);
			}
			// would be nice to complain here, but we lack key's location
			return 0;
		}
		return ip;
	}
	return 0;
}

PglRec *PglRec::clone() const {
	PglRec *rec = new PglRec;

	for (int i = 0; i < count(); ++i) {
		if (valAt(i)) {
			SynSymTblItem *clone = new SynSymTblItem(*valAt(i));

			if (SynSym *s = valAt(i)->sym())
				clone->sym(s->clone(s->type()));

			rec->add(keyAt(i), clone);
		}
	}
	return rec;
}

SynSymTblItem *PglRec::bAdd(const String &type, const String &name, SynSym *s) {
	SynSymTblItem *member = new SynSymTblItem(type, name);
	member->sym(s);
	add(member->name(), member);
	return member;
}
