
/*
 * $Id: PglSizeSym.cc,v 1.6.6.2 1999/10/31 03:10:05 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include <math.h>

#include "PglNumSym.h"
#include "PglTimeSym.h"
#include "PglBwidthSym.h"
#include "PglSizeSym.h"


SizeSym::SizeSym(const BigSize &aVal): ExpressionSym("size"), theVal(aVal) {
}

bool SizeSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Size" || type == "size";
}


SynSym *SizeSym::clone(const String &type) const {
	if (isA(type))
		return new SizeSym(theVal);
	return ExpressionSym::clone(type);
}

ExpressionSym *SizeSym::unOper(const String &op) const {
	if (*op == '+')
		return new SizeSym(+theVal);
	else
	if (*op == '-')
		return new SizeSym(-theVal);
	else
		return ExpressionSym::unOper(op);
}


ExpressionSym *SizeSym::bnOper(const String &op, const SynSym &s) const {
	if (op.len() != 1)
		return ExpressionSym::bnOper(op, s);

	if (SizeSym *ss = (SizeSym*)s.clone("size"))
		return operSS(op, ss, s);

	if (TimeSym *ts = (TimeSym*)s.clone("time"))
		return operST(op, ts, s);

	if (BwidthSym *rs = (BwidthSym*)s.clone("bwidth"))
		return operSB(op, rs, s);

	if (NumSym *ns = (NumSym*)s.clone("float"))
		return operSN(op, ns, s);
	
	return ExpressionSym::bnOper(op, s);
}

ExpressionSym *SizeSym::operSS(const String &op, SizeSym *ss, const SynSym &s) const {
	BigSize otherVal = ss->val();
	delete ss;
	
	if (*op == '/')
		checkDenom(otherVal != 0);

	switch (*op) {
		case '+': return new SizeSym(theVal + otherVal);
		case '-': return new SizeSym(theVal - otherVal);
		case '/': return new NumSym(theVal / otherVal);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operST(const String &op, TimeSym *ts, const SynSym &s) const {
	Time otherInterval = ts->val();
	delete ts;

	switch (*op) {
		case '/': return new BwidthSym(Bandwidth(theVal, otherInterval));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operSB(const String &op, BwidthSym *bs, const SynSym &s) const {
	Bwidth otherVal = bs->val();
	delete bs;
	
	if (*op == '/')
		checkDenom(otherVal);

	switch (*op) {
		case '/': return new TimeSym(Time::Secd(theVal.byted()/BytesPerSec(otherVal)));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operSN(const String &op, NumSym *ns, const SynSym &s) const {
	double otherFactor = ns->val();
	delete ns;
	
	if (*op == '/')
		checkDenom(otherFactor);

	switch (*op) {
		case '*': return new SizeSym(theVal*otherFactor);
		case '/': return new SizeSym(theVal/otherFactor);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &SizeSym::print(ostream &os, const String &) const {
	return os << theVal;
}
