
/*
 * $Id: SocketSym.cc,v 1.5 1999/09/07 17:47:53 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglBoolSym.h"
#include "PglRec.h"
#include "PglTimeSym.h"
#include "SocketSym.h"


SocketSym::SocketSym(): RecSym("Socket", new PglRec) {

	// add fields

	theRec->bAdd("bool", "nagle", new BoolSym(false));
	theRec->bAdd("time", "linger_tout", 0);
}

SocketSym::SocketSym(String const &aType, PglRec *aRec):
	RecSym(aType, aRec) {
}

bool SocketSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Socket";
}

SynSym *SocketSym::clone(const String &type) const {
	if (isA(type))
		return new SocketSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

bool SocketSym::lingerTout(Time &tout) const {
	SynSymTblItem *touti = 0;
	Assert(theRec->find("linger_tout", touti));
	if (touti->sym())
		tout = ((const TimeSym&)touti->sym()->cast("time")).val();
	return touti->sym();
}

bool SocketSym::nagle(bool &doNagle) const {
	SynSymTblItem *ni = 0;
	Assert(theRec->find("nagle", ni));
	if (ni->sym())
		doNagle = ((const BoolSym&)ni->sym()->cast("bool")).val();
	return ni->sym();
}
