
/*
 * $Id: SynSym.h,v 1.5 1999/09/26 01:25:59 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XPARSER_SYNTAX_SYMBOL_H
#define XPARSER_SYNTAX_SYMBOL_H

#include "LexToken.h"

// The SynSym class is the base class for all attributes
// stack can be found on the generated parser's stack.

class SynSym {
	public:
		SynSym(const String &aType);
		virtual ~SynSym();

		const String &type() const { return theType; }
		virtual bool isA(const String &type) const;
		bool canBe(const String &type) const;
		bool equal(const SynSym &s) const;

		// aborts if incompatible types
		const SynSym &cast(const String &type) const;
		SynSym &cast(const String &type);

		// creates a copy of itself or compatible type
		// returns 0 if incompatible types and by default
		virtual SynSym *clone(const String &type) const;
		SynSym *dup() const { return clone(type()); }

		const TokenLoc &loc() const { return theLoc; }
		void loc(const TokenLoc &aLoc) { theLoc = aLoc; }

		virtual ostream &print(ostream &os, const String &pfx = 0) const;

	protected:
		void type(const String &aType);

	protected:
		TokenLoc theLoc;

	private:
		String theType;
};

inline ostream &operator <<(ostream &os, const SynSym &sym) { return sym.print(os); }

#endif
