
/*
 * $Id: Future.h,v 1.3 1999/09/07 17:48:38 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef FUTURE_CLASS_H
#define FUTURE_CLASS_H

#include "Array.h"

// "Future plan": a circular buffer; new entries are inserted
// at random positions relative to "now"; "now" or "next step"
// is at position "zero"
// for simplicity, assume that at most MAX_INT items are inserted
// over the life time of an object
template <class Item>
class Future {
	public:
		Future(int aCapacity = 0): theLog(aCapacity), theOffset(0) {}

		void capacity(int aCapacity) { Assert(!theOffset); theLog.stretch(aCapacity); }
		int capacity() const { return theLog.capacity(); }

		const Item &operator [](int off) const { return theLog[(theOffset + off) % theLog.capacity()]; }
		Item &operator [](int off) { return theLog[(theOffset + off) % theLog.capacity()]; }
		void shift(Item i) { theLog[theOffset++ % theLog.capacity()] = i; }

	protected:
		Array<Item> theLog;
		int theOffset;      // "now" or the start of a plan
};

#endif
